import java.util.List;
import java.util.Random;

/**
 * Jednoduchy model králíka.
 * Kralik stárne, pohybuje se, množí se a umírá.
 * 
 * @author David J. Barnes and Michael Kolling
 * @author Lubos Pavlicek
 * @version 2005.10.23
 */
public class Kralik
{
    // Charakteristiky všech králiků (statické položky).

    // Věk, ve kterém se králík začíná množit.
    private static final int BREZI_VEK = 5;
    // Maximální stáří králíka.
    private static final int MAX_VEK = 50;
    // Pravděpodobnost, že králík zabřezne.
    private static final double PRAVDEPODOBNOST_ZABREZNUTI = 0.15;
    // Maximální počet mláďat.
    private static final int MAX_MLADAT = 5;
    // Sdílený generátor náhodných čísel.
    private static final Random rand = new Random();
    
    // Individuální vlastnosti králíka (datové atributy).
    
    // Stáří králíka.
    private int vek;
    // Zda je králík živý.
    private boolean zivy;
    // Pozice králíka
    private Pozice pozice;

    /**
     * Vytvoření králíka. Králík může být vytvořen se stářím 0 - nově narození králíci, 
     * nebo s náhodným věkem (při počáteční inicializaci)
     * 
     * @param nahodnyVek Pokud true, přiřadí se králíkovi náhodné stáří.
     */
    public Kralik(boolean nahodnyVek) {
        vek = 0;
        zivy = true;
        if (nahodnyVek) {
            vek = rand.nextInt(MAX_VEK);
        }
    }
    
    /**
     * To co většinou dělá králík - běhá kolem. Občas porodí mladé, někdy zemře stářím.
     *
     * @param novaLouka       Louka, na které králík bude běhat po tomto kroku. 
     * @param narozeniKralici Seznam nově narozených králíků.
     */
    public void behat(Louka louka, List<Kralik> narozeniKralici) {
        zvysVek();
        if(zivy) {
            int mladat = porod();
            for(int b = 0; b < mladat; b++) {
                Kralik novyKralik = new Kralik(false);
                Pozice poz = louka.volnaSousedniPozice(pozice);
                if (poz != null) {   // pokud není místo, tak králik hned umře
                    narozeniKralici.add(novyKralik);
                    novyKralik.setPozice(poz);
                    louka.vloz(novyKralik, poz);
                }
            }
            Pozice novaPozice = louka.volnaSousedniPozice(pozice);
            
            if(novaPozice != null) {
                louka.presun(this, pozice, novaPozice);
                setPozice(novaPozice);
            }
            else {
                // pokud jsou všechny pozice obsazeny, tak zemře
                louka.odeberZvire(pozice);
                zivy = false;
            }
        }
        else { // králík je mrtvý
            louka.odeberZvire(pozice);
        }
    }
    
    /**
     * Zvysi vek.
     * Pokud je králik moc starý, tak zemře.
     */
    private void zvysVek() {
        vek++;
        if(vek > MAX_VEK) {
            zivy = false;
        }
    }
    
    /**
     * Vrací počet mláďat králíka, pokud může zabřeznout.
     * @return Počet mláďat (může být i nula).
     */
    private int porod() {
        int mladat = 0;
        if (vek >= BREZI_VEK && rand.nextDouble() <= PRAVDEPODOBNOST_ZABREZNUTI) {
            mladat = rand.nextInt(MAX_MLADAT) + 1;
        }
        return mladat;
    }
    
    /**
     * Zda je kralik ještě živý.
     * @return true pokud je králík ještě živý.
     */
    public boolean jeZivy() {
        return zivy;
    }

    /**
     * Sdělíme králíkovi, že je snězen, tj. mrtvý :(
     */
    public void snezen() {
        zivy = false;
    }
    
    /**
     * Nastaveni pozice králíka.
     * @param pozice Pozice králíka.
     */
    public void setPozice(Pozice pozice) {
        this.pozice = pozice;
    }
}
