import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;

/**
 * Třída reprezentuje louku rozdělenou jednotlivá políčka.
 * Na každé pozici může být umístěno jedno zvíře.
 * 
 * @author David J. Barnes and Michael Kolling
 * @author Lubos Pavlicek
 * @version 2005.10.23
 */
public class Louka
{
    private static final Random rand = new Random();
    
    // délka a šířka louky.
    private int delka, sirka;
    // Louka, na které jsou zvířata.
    private Object[][] louka;
    // Seznam pozic
    private List<Pozice> seznamPozic;

    /**
     * Reprezentuje louku daných rozměrů.
     * @param delka Délka louky.
     * @param sirka Šířka louky.
     */
    public Louka(int delka, int sirka)
    {
        this.delka = delka;
        this.sirka = sirka;
        louka = new Object[delka][sirka];
        seznamPozic = new LinkedList<Pozice>();
        for (int radek=0; radek < delka; radek++) {
            for (int sloupec=0; sloupec < sirka; sloupec++) {
                seznamPozic.add(new Pozice(radek,sloupec));
            }
        }

    }
    
    /**
     * Výmaz louky (zrušení všech zvířat).
     */
    public void clear()
    {
        for(int radek = 0; radek < delka; radek++) {
            for(int sloupec = 0; sloupec < sirka; sloupec++) {
                louka[radek][sloupec] = null;
            }
        }
    }
    
    /**
     * Umístí králíka na danou pozici.
     * Pokud již na této pozici je nějaké zvíře, tak se ztratí.
     * @param kralik    Králík, který se ma umístit.
     * @param pozice    Pozice, na kterou se má králík umístit.
     */
    public void vloz(Kralik kralik, Pozice pozice)
    {
        louka[pozice.getRadek()][pozice.getSloupec()] = kralik;
        kralik.setPozice(pozice);
    }

    /**
     * Umístí lišku na danou pozici.
     * Pokud již na této pozici je nějaké zvíře, tak se ztratí.
     * @param kralik    Liška, která se má umístit.
     * @param pozice    Pozice, na kterou se má liška umístit.
     */
    public void vloz(Liska liska, Pozice pozice) {
        louka[pozice.getRadek()][pozice.getSloupec()] = liska;
        liska.setPozice(pozice);
    }

    /**
     * Přesune lišku z jedné pozice na druhou.
     * @param liska         liška, která se má přesunout
     * @param pozice        Pozice, ze které se má zvíře přesunout.
     * @param novaPozice    Pozice, na kterou se má zvíře umístit.
     */
    public void presun(Liska liska, Pozice pozice, Pozice novaPozice) {
            if (pozice.equals(novaPozice)) {
                return;
            }
            if (louka[novaPozice.getRadek()][novaPozice.getSloupec()] == null) {
                louka[novaPozice.getRadek()][novaPozice.getSloupec()] = 
                                louka[pozice.getRadek()][pozice.getSloupec()];
                louka[pozice.getRadek()][pozice.getSloupec()] = null;
                liska.setPozice(novaPozice);
            }
            else {
                System.out.println("presouvá se na obsazenou pozici");
            }
    }

    /**
     * Přesune králíka z jedné pozice na druhou.
     * @param králík        králík, která se má přesunout
     * @param pozice        Pozice, ze které se má zvíře přesunout.
     * @param novaPozice    Pozice, na kterou se má zvíře umístit.
     */
    public void presun(Kralik kralik, Pozice pozice, Pozice novaPozice) {
            if (pozice.equals(novaPozice)) {
                return;
            }
            if (louka[novaPozice.getRadek()][novaPozice.getSloupec()] == null) {
                louka[novaPozice.getRadek()][novaPozice.getSloupec()] = 
                                louka[pozice.getRadek()][pozice.getSloupec()];
                louka[pozice.getRadek()][pozice.getSloupec()] = null;
                kralik.setPozice(novaPozice);
            }
            else {
                System.out.println("presouvá se na obsazenou pozici");
            }
    }

    /**
     * Odebere zvíře z danné pozice.
     * @param pozice    Pozice, ze které se má zvíře odebrat.
     */
    public void odeberZvire(Pozice pozice) {
        louka[pozice.getRadek()][pozice.getSloupec()] = null;
    }
    
    /**
     * Return the animal at the given location, if any.
     * @param location Where in the louka.
     * @return The animal at the given location, or null if there is none.
     */
    public Object getObjectAt(Pozice location)
    {
        return getObjectAt(location.getRadek(), location.getSloupec());
    }
    
    /**
     * Vrátí zvíře na zadané pozici.
     * @param radek     Řádek pozice.
     * @param sloupec   Sloupec pozice.
     * @return   Vrátí zvíře na zadané pozici nebo null, pokud na pozici žádné zvíře není.
     */
    public Object getObjectAt(int radek, int sloupec)
    {
        return louka[radek][sloupec];
    }
    
    /**
     * Vrací náhodnou sousední pozici od zadané pozice.
     * Vrácená pozice je uvnitř hranic louky.
     * 
     * @param  pozice   Pozice, od které se hledají sousední pozice.
     * @return          Platnou pozici z dané oblasti. Může to být i vlastní zadaná pozice.
     */
    public Pozice nahodnaSousedniPozice(Pozice pozice) {
        List <Pozice> seznam = sousedniPozice(pozice);
        seznam.add(pozice);
        return seznam.get(rand.nextInt(seznam.size()));
    }
    
    /**
     * Vrací některou ze sousedních volných pozic. Pokud není žádná sousední pozice
     * volná, vrátí aktuální pozici - pokud ani ta není volná, tak vrací hodnotu null.
     * Vrácená pozice je uvnitř louky.
     * @param  pozice   Pozice, kolem které se hledá volné místo.
     * @return          Pozici uvnitř louky sousedící se zadanou pozicí, na které není
     *                  žádné zvíře. Může to být i zadaná pozice (střed oblasti). Pokud
     *                  žádná z těchto pozic není volná, vrátí hodnotu null.
     */
    public Pozice volnaSousedniPozice(Pozice pozice) {
        for (Pozice sousedni : sousedniPozice(pozice)) {
            if(louka[sousedni.getRadek()][sousedni.getSloupec()] == null) {
                return sousedni;
            }
        }
        // kontroluji zadanou pozici, zda je volná
        if(louka[pozice.getRadek()][pozice.getSloupec()] == null) {
            return pozice;
        } 
        else {
            return null;
        }
    }

    /**
     * Vrací seznam náhodně setříděný seznam sousedních míst od zadané pozice.
     * Seznam neobsahuje vlastní pozici (střed oblasti).
     * Všechny pozice v seznamu leží na louce.
     * @param  pozice   Pozice, kolem které se hledají sousední pozice.
     * @return          Seznam (List) se sousedními pozicemi.
     */
    public List<Pozice> sousedniPozice(Pozice pozice)
    {
        int radek   = pozice.getRadek();
        int sloupec = pozice.getSloupec();
        List<Pozice> seznam = new LinkedList<Pozice>();
        for (int rOffset = -1; rOffset <= 1; rOffset++) {
            int dalsiRadek = radek + rOffset;
            if(dalsiRadek >= 0 && dalsiRadek < delka) {
                for(int sOffset = -1; sOffset <= 1; sOffset++) {
                    int dalsiSloupec = sloupec + sOffset;
                    // vynechávám též zadanou pozici.
                    if(dalsiSloupec >= 0 && dalsiSloupec < sirka && (rOffset != 0 || sOffset != 0)) {
                        seznam.add(new Pozice(dalsiRadek, dalsiSloupec));
                    }
                }
            }
        }
        Collections.shuffle(seznam,rand);
        return seznam;
    }
    
    /**
     * Vrací seznam všech pozic na louce.
     * @return    Seznam všech pozic na louce.
     */
    public List<Pozice> seznamPozic() {
        return seznamPozic;
    }

    /**
     * @return Délka louky.
     */
    public int getDelka()
    {
        return delka;
    }
    
    /**
     * @return Šířka louky.
     */
    public int getSirka()
    {
        return sirka;
    }
}
