import java.util.HashMap;
import java.util.Map;

/**
 * Tato třída sbírá a poskytuje statistické údaje o louce.
 * Je flexibilní - vytváří čítač pro každý typ umístěný na 
 * louce.
 * 
 * @author David J. Barnes and Michael Kolling
 * @author Lubos Pavlicek
 * @version 2005.10.23
 */
public class LoukaStats {
    
    // Čítače pro každý typ na louce (liška, králík, ...) při simulaci.
    // Jako value je pomocný typ Citac, neboť je efektivnější než např. Integer.
    private Map<Class, Citac> citace;
    // Zda jsou počty aktuální.
    private boolean citaceOK;

    /**
     * V konstruktoru se vytvářejí potřebné instance
     */
    public LoukaStats() {
        citace = new HashMap<Class, Citac>();
        citaceOK = true;
    }

    /**
     * @return Řetězec popisující zvířata na louce.
     */
    public String getDetailyOPopulaci(Louka louka) {
        StringBuilder buffer = new StringBuilder();
        if(!citaceOK) {
            spoctiZvirata(louka);
        }
        for (Class trida : citace.keySet()) {
            buffer.append(trida.getName());
            buffer.append(": ");
            buffer.append(citace.get(trida));
            buffer.append(' ');
        }
        return buffer.toString();
    }
    
    /**
     * Resetuje se stav čítačů - nastaví se jejich obsah na nulu.
     */
    public void reset() {
        citaceOK = false;
        for (Citac citac : citace.values()) {
            citac.nuluj();
        }
    }

    /**
     * Zvýší počet o jedna u čítače příslušejícího ke třídě (k typu).
     */
    public void zvysCitac(Class animalClass) {
        Citac citac;
        if(citace.containsKey(animalClass)) {
            citac = citace.get(animalClass);
        }
        else {
            // zatím není čítač pro tento druh
            citac = new Citac();
            citace.put(animalClass, citac);
        }
        citac.pricti();
    }

    /**
     * Byl dokončen součet zvířat na louce.
     */
    public void spocteno() {
        citaceOK = true;
    }

    /**
     * Určí, zda má ještě smysl pokračovat v simulaci.
     * 
     * @param  louka Louka, pro kterou se spočítají jednotlivé druhy
     * @return true Pokud je na louce více než jeden druh.
     */
    public boolean pokracovat(Louka louka) {
        // Kolik čítačů je nenulových.
        int nonZero = 0;
        if(!citaceOK) {
            spoctiZvirata(louka);
        }
        for (Citac citac : citace.values()) {
            if (citac.getPocet() > 0) {
                nonZero++;
            }
        }
        return nonZero > 1;
    }
    
    /**
     * Spočítá králíky a lišky na louce.
     * Počty se neudržují v okamžiku rození/úmrtí zvířat,
     * ale počítají se na požádání.
     */
    private void spoctiZvirata(Louka louka) {
        reset();
        for(int radek = 0; radek < louka.getDelka(); radek++) {
            for(int sloupec = 0; sloupec < louka.getSirka(); sloupec++) {
                Object zvire = louka.getObjectAt(radek, sloupec);
                if(zvire != null) {
                    zvysCitac(zvire.getClass());
                }
            }
        }
        citaceOK = true;
    }

    /**
     * Vnitřní třída pro počítání instancí. Počítání lze sice udělat i pomocí toho, že
     * v mapě citace jako hodnota může být typ Integer, ale použití pomocné třídy je efektivnější,
     * neboť se vytváří mnohem méně instancí.
     */
    
    private class Citac {
        private int pocet=0;
        public void pricti() {
            pocet++;
        }
        public int getPocet() {
            return pocet;
        }
        public String toString() {
            return Integer.toString(pocet);
        }
        public void nuluj() {
            pocet=0;
        }
    }
}
